package agent;

import java.util.Vector;

/** The top-level class for an agent simulation. This can be used for
    either single or multi-agent environments. */
public abstract class Environment {

  protected Vector agents = null;
  protected State state = null;

  /** Construct a new environment. Initialize the agents vector. */
  public Environment() {

    agents = new Vector();
  }

  /** Add a new agent to the environment. */
  public abstract void addAgent(Agent agent);

  /** Run the simulation starting from a given state. This consists of
      a sense-act loop for each agent. An alternative approach would be
      to allow the agent to decide when it will sense and act, but that
      would probably require multiple threads. */
  public void start(State initState) {

    int numAgents;
    Percept p;
    Action action;
    Agent a;
    
    state = initState;
    state.display();

    a = (Agent)agents.firstElement();
    numAgents = agents.size();
    while (!isComplete()) {
      for (int i=0; i < numAgents; i++) {
        a = (Agent)agents.elementAt(i);
        p = getPercept(a);
        a.see(p);
        action = a.selectAction();
        updateState(a, action);
        state.display();
      }
    }
  }

  /** Is the simulation over? Returns true if it is, otherwise false. */
  protected abstract boolean isComplete();

  /** Create a percept for an agent. This should implement the
      see: S -> P function. */
  protected abstract Percept getPercept(Agent a);

  /** Execute an agent's action and update the environment's state. This
   implements the env: S x A -> S function. */
  protected void updateState(Agent a, Action action) {

    state = action.execute(a, state);
  }


}



